/***********************************************************/
//  BeUAE - The Be Un*x Amiga Emulator
//
//  BeOS port - gui settings
//
//  (c) 2001 Axel Doerfler
//
/***********************************************************/

#include <Application.h>
#include <Window.h>
#include <Resources.h>
#include <experimental/ResourceSet.h>

#include <ctype.h>
#include <string.h>

#include "be-Settings.h"
#include "be-FileSystemSettings.h"

#include <VGroup.h>
#include <HGroup.h>
#include <MBorder.h>
#include <MScrollView.h>
#include <MListView.h>
#include <MTextControl.h>
#include <MStringView.h>
#include <MCheckBox.h>
#include <MPopup.h>
#include <MButton.h>
#include <Space.h>
#include "santa/BetterScrollView.h"

extern "C" {
#include "sysconfig.h"
#include "sysdeps.h"
#include "config.h"
#include "options.h"
#include "custom.h"
#include "disk.h"
#include "autoconf.h"
#include "uae.h"
};

// filesystem & hardfiles
const int kFileSystemVolumeWidth = 75;
const int kFileSystemReadOnlyWidth = 70;
const int kFileSystemTypeWidth = 35;

// external variables
extern BBitmap *gLocalDirectoryBitmap,*gHardFileBitmap;


/*************************************************************************************/
//  a window to add/edit a local directory to the filesystems
//	#pragma mark -


LocalDirectoryWindow::LocalDirectoryWindow(BRect frame, BListView *list, LocalDirectoryItem *item)
	: MWindow(frame,NULL,B_MODAL_WINDOW,0)
{
	MView *topView = new HGroup
	(
		new Space(kHorizontal),
		new VGroup
		(
			minimax(300,50,1e6,1e6),
			new Space(kVertical),
			new MBorder
			(
				M_LABELED_BORDER,kBorderSpacing,(char *)(item != NULL ? "Edit a file system" : "Add a new file system"),
				new VGroup
				(
					fVolume = new MTextControl("Volume Name:",(char *)(item != NULL ? item->fVolume : "")),
					createFileView(&fRoot,"Local Directory:",kMsgFileSystemRoot),
					fAccessMode = new MPopup("Access Mode:",new BMessage(kMsgFileSystemAccessMode),NULL,
						"read only","read/write",NULL),
					NULL
				)
			),
			new Space(minimax(0,6,0,1e6)),
			new HGroup
			(
				new Space(),
				new MButton("Ok",new BMessage(kMsgFileSystemOk)),
				new MButton("Cancel",new BMessage(B_QUIT_REQUESTED)),
				NULL
			),
			new Space(kVertical),
			NULL
		),
		new Space(kHorizontal),
		NULL
	);
	AddChild(dynamic_cast<BView*>(topView));

	if (item != NULL)
	{
		fRoot->SetText(item->fRoot);
		fAccessMode->SetActive(item->fReadOnly ? 0 : 1);
	}
	fListView = list;
	fItem = item;

	fFilePanel = new BFilePanel(B_OPEN_PANEL,new BMessenger(this),NULL,B_DIRECTORY_NODE,false,NULL,NULL,true);
	fFilePanel->Window()->SetTitle("Select the root directory of the File System");
}


void LocalDirectoryWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what)
	{
		case kMsgFileSystemRoot:
		{
			char *text = (char *)fRoot->Text();
		
			BPath path(text);
			if (path.InitCheck() == B_NO_ERROR)
				if (path.GetParent(&path) == B_NO_ERROR)
					fFilePanel->SetPanelDirectory(path.Path());
		
			fFilePanel->Show();
			break;
		}
		case B_REFS_RECEIVED:
		{
			entry_ref ref;
			
			if (msg->FindRef("refs", &ref) == B_NO_ERROR)
			{
				BEntry entry(&ref);
				if (entry.InitCheck() == B_NO_ERROR)
				{
					BPath path;
					entry.GetPath(&path);
		
					fRoot->SetText(path.Path());
					if (strlen(fVolume->Text()) == 0)
						fVolume->SetText(path.Leaf());
				}
			}
			break;
		}
		case kMsgFileSystemAccessMode:
			fReadOnly = popUpIndex(msg) == 1 ? false : true;
			break;
		case kMsgFileSystemOk:
		{
			BWindow *window = fListView->Window();
			
			if (window->Lock())
			{
				if (fItem != NULL)
				{
					fItem->fReadOnly = fReadOnly;
					free(fItem->fVolume);  free(fItem->fRoot);
					fItem->fVolume = strdup(fVolume->Text());
					fItem->fRoot = strdup(fRoot->Text());
				}
				else	// create new
					fListView->AddItem(new LocalDirectoryItem((char *)fVolume->Text(),(char *)fRoot->Text(),fReadOnly));
	
				fListView->Invalidate();
				window->Unlock();
			}
			Quit();
			break;
		}
	}
}


/*************************************************************************************/
//  a window to add/edit a hardfile to the filesystems
//	#pragma mark -


HardFileWindow::HardFileWindow(BRect frame,BListView *list,HardFileItem *item)
	: MWindow(frame,NULL,B_MODAL_WINDOW,0)
{
	MView *topView = new HGroup
	(
		new Space(kHorizontal),
		new VGroup
		(
			minimax(300,50,1e6,1e6),
			new Space(kVertical),
			new MBorder
			(
				M_LABELED_BORDER,kBorderSpacing,(char *)(item != NULL ? "Edit a hardfile" : "Add a new hardfile"),
				new VGroup
				(
					createFileView(&fFile,"File:",kMsgFileSystemRoot),
					fAccessMode = new MPopup("Access Mode:",new BMessage(kMsgFileSystemAccessMode),NULL,
						"read only","read/write",NULL),
					new Space(kVertical),
					new HGroup
					(
						fSectorsPerTrack = new MTextControl("Sectors per Track:",(char *)(item != NULL ? "" : "32")),
						new Space(kHorizontal),
						fSurfaces = new MTextControl("Surfaces:",(char *)(item != NULL ? "" : "1")),
						NULL
					),
					new Space(kVertical),
					new HGroup
					(
						fReservedBlocks = new MTextControl("Reserved Blocks:",(char *)(item != NULL ? "" : "2")),
						new Space(kHorizontal),
						fBlockSize = new MTextControl("Block Size:",(char *)(item != NULL ? "" : "512")),
						NULL
					),
					new Space(kVertical),
					fSize = new MTextControl("Hardfile Size:",""),
					new Space(kVertical),
					new MButton("Create Hardfile",new BMessage(kMsgCreateHardFile)),
					NULL
				)
			),
			new Space(minimax(0,6,0,1e6)),
			new HGroup
			(
				new Space(),
				new MButton("Ok",new BMessage(kMsgFileSystemOk)),
				new MButton("Cancel",new BMessage(B_QUIT_REQUESTED)),
				NULL
			),
			new Space(kVertical),
			NULL
		),
		new Space(kHorizontal),
		NULL
	);
	AddChild(dynamic_cast<BView*>(topView));

	DivideSame(fSectorsPerTrack,fReservedBlocks,NULL);
	DivideSame(fSurfaces,fBlockSize,NULL);

	if (item != NULL)
	{
		fFile->SetText(item->fFile);
		fAccessMode->SetActive(item->fReadOnly ? 0 : 1);
		
		char t[256];

		sprintf(t,"%d",item->fBlockSize);
		fBlockSize->SetText(t);
		
		sprintf(t,"%d",item->fSectorsPerTrack);
		fSectorsPerTrack->SetText(t);
		
		sprintf(t,"%d",item->fSurfaces);
		fSurfaces->SetText(t);
		
		sprintf(t,"%d",item->fReservedBlocks);
		fReservedBlocks->SetText(t);
	}
	fListView = list;
	fItem = item;

	fFilePanel = new BFilePanel(B_OPEN_PANEL,new BMessenger(this),NULL,B_FILE_NODE,false,NULL,NULL,true);
	fFilePanel->Window()->SetTitle("Select a Hardfile");
}


void HardFileWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what)
	{
		case kMsgFileSystemRoot:
		{
			char *text = (char *)fFile->Text();
		
			BPath path(text);
			if (path.InitCheck() == B_NO_ERROR)
				if (path.GetParent(&path) == B_NO_ERROR)
					fFilePanel->SetPanelDirectory(path.Path());
		
			fFilePanel->Show();
			break;
		}
		case B_REFS_RECEIVED:
		{
			entry_ref ref;
			
			if (msg->FindRef("refs", &ref) == B_NO_ERROR)
			{
				BEntry entry(&ref);
				if (entry.InitCheck() == B_NO_ERROR)
				{
					BPath path;
					entry.GetPath(&path);
		
					fFile->SetText(path.Path());
					
					struct stat statInfo;
					
					if (entry.Exists() && entry.GetStat(&statInfo) == B_OK)
					{
						char t[64];
						
						MemorySlider::MemoryText(t,statInfo.st_size);
						fSize->SetText(t);
					}
				}
			}
			break;
		}
		case kMsgCreateHardFile:
			CreateHardFile();
			break;
		case kMsgFileSystemAccessMode:
			fReadOnly = popUpIndex(msg) == 1 ? false : true;
			break;
		case kMsgFileSystemOk:
		{
			BWindow *window = fListView->Window();
			int sectorsPerTrack,surfaces,reservedBlocks,blockSize;
			
			sectorsPerTrack = atoi(fSectorsPerTrack->Text());
			surfaces = atoi(fSurfaces->Text());
			reservedBlocks = atoi(fReservedBlocks->Text());
			blockSize = atoi(fBlockSize->Text());

			if (window->Lock())
			{
				if (fItem != NULL)
				{
					fItem->fReadOnly = fReadOnly;
					free(fItem->fFile);
					fItem->fFile = strdup(fFile->Text());
				}
				else	// create new
					fListView->AddItem(new HardFileItem((char *)fFile->Text(),fReadOnly,sectorsPerTrack,surfaces,reservedBlocks,blockSize));
	
				fListView->Invalidate();
				window->Unlock();
			}
			Quit();
			break;
		}
	}
}


status_t HardFileWindow::Alert(char *text,char *button1,alert_type type)
{
	BAlert *alert = new BAlert("BeUAE - Create hardfile",text,"Ok",button1,NULL,B_WIDTH_AS_USUAL,type);
	
	if (alert != NULL)
		return alert->Go();
	
	return -1;
}


off_t HardFileWindow::ConvertSize(const char *text)
{
	char *suffix;
	double size = strtod(text,&suffix);
	
	for(;*suffix && isspace(*suffix);suffix++);
	
	if (!strcasecmp(suffix,"gb"))
		size *= kGBSize;
	else if (!strcasecmp(suffix,"mb"))
		size *= kMBSize;
	else if (!strcasecmp(suffix,"kb"))
		size *= kKBSize;
	
	return (off_t)(size+0.5);
}


void HardFileWindow::CreateHardFile()
{
	const char *filename = fFile->Text();
	uint64 size = ConvertSize(fSize->Text());
	
	if (filename == NULL || *filename == 0)
	{
		Alert("No file name specified!");
		return;
	}
	if (size < kMBSize)	// hardfiles smaller than 1 MB
	{
		Alert("The size of the hardfile is too small! 1 MB is the minimal size.");
		return;
	}
	if (size > 2*kGBSize)	// hardfiles greater than 2 GB
	{
		if (Alert("The size of the hardfile exceeds 2 GB! It will be shrinked to 2 GB.","Cancel"))
			return;
		size = 2*kGBSize;
	}
	
	BEntry entry(filename);
	
	if (entry.Exists())
	{
		if ((new BAlert("BeUAE - Create hardfile","The file specified does already exist!","Replace","Cancel",NULL,B_WIDTH_AS_USUAL,B_WARNING_ALERT))->Go())
			return;
	}

	BFile file(&entry,B_WRITE_ONLY | B_CREATE_FILE | B_ERASE_FILE);
	status_t error;
	if ((error = file.InitCheck()) == B_OK)
	{
		if ((error = file.SetSize((off_t)size)) == B_OK)
		{
			Alert("Creating the hardfile has succeded!",NULL,B_INFO_ALERT);

			/* update text controls */
			return;
		}
	}
	char text[256] = "Creating the hardfile has failed:\n\n\t";
	
	strncat(text,strerror(error),255);
	text[255] = 0;
	Alert(text,NULL,B_STOP_ALERT);
}


/*************************************************************************************/
//  the label of the filesystem list
//	#pragma mark -


FileSystemLabel::FileSystemLabel(BRect(frame))
	: BView(frame,NULL,B_FOLLOW_RIGHT | B_FOLLOW_TOP,B_WILL_DRAW)
{
	ResizeTo(frame.Width(),14);
	SetLowColor(ui_color(B_PANEL_BACKGROUND_COLOR));
}


void FileSystemLabel::DrawBox(BRect rect,char *label)
{
	PushState();
	
	SetHighColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	BRect inner(rect);
	inner.left++;  inner.top++;  inner.right -= 2;  inner.bottom -= 2;
	FillRect(inner);

	SetHighColor(tint_color(HighColor(),B_DARKEN_1_TINT));
	StrokeLine(BPoint(rect.left+1,rect.bottom-1),BPoint(rect.right-1,rect.bottom-1));
	StrokeLine(BPoint(rect.right-1,rect.top));

	SetHighColor(tint_color(HighColor(),B_DARKEN_2_TINT));
	StrokeLine(BPoint(rect.left,rect.bottom),BPoint(rect.right,rect.bottom));
	StrokeLine(BPoint(rect.right,rect.top));

	if (label != NULL)
	{
		SetHighColor(0,0,0);
		SetFontSize(10);
		DrawString(label,BPoint(rect.left+6,rect.bottom-4));
	}
	PopState();
}


void FileSystemLabel::Draw(BRect updateRect)
{
	BRect itemRect(0,0,Bounds().Width(),Bounds().Height());
	BRect rect(itemRect);

	rect.right = rect.left+kFileSystemTypeWidth;
	DrawBox(rect,"Type");

	rect.left = rect.right+1;	rect.right = rect.left+kFileSystemVolumeWidth;
	DrawBox(rect,"Volume Name");
	
	rect.left = rect.right+1;
	rect.right = itemRect.right - kFileSystemReadOnlyWidth;
	DrawBox(rect,"Local Directory/Hardfile");
	
	rect.left = rect.right+1;  rect.right = itemRect.right;
	DrawBox(rect,"Access Mode");
}


/*************************************************************************************/
//  the filesystem list; it's partly based on Brian Tietz' ColumnListView
//	#pragma mark -


FileSystemList::FileSystemList(BRect frame)
	: BListView(frame,NULL,B_SINGLE_SELECTION_LIST,B_FOLLOW_ALL_SIDES)
{
	BRect labelFrame(frame.left,frame.top,frame.right,frame.top+20);
	fLabel = new FileSystemLabel(labelFrame);

	fScrollView = new BetterScrollView(NULL,this,B_FOLLOW_ALL_SIDES,B_WILL_DRAW | B_FRAME_EVENTS,false,true,false,B_FANCY_BORDER);

	labelFrame = fLabel->Frame();
	BRect updatedFrame = Frame();

	// make room for the label
	ResizeTo(Frame().Width(),Frame().Height()-labelFrame.Height()-1.0);
	MoveTo(updatedFrame.left,updatedFrame.top+labelFrame.Height()+1.0);
	fLabel->MoveTo(updatedFrame.left,updatedFrame.top);

	// add the label view
	fScrollView->AddChild(fLabel);

	//Remove and re-add the BListView so that it will draw after the label
	fScrollView->RemoveChild(this);
	fScrollView->AddChild(this);

	UpdateScrollBars();
}


FileSystemList::~FileSystemList()
{
}


void FileSystemList::UpdateScrollBars()
{
	float height = 0.0;
	int32 num = CountItems();

	for(int32 i = 0;i < num;i++)
		height += ItemAt(i)->Height()+1.0;
	if(num > 0)
		height -= 1.0;

	// update the scroll bars
	fScrollView->SetDataRect(BRect(0,0,fScrollView->Bounds().Width(),height),true);
}


void FileSystemList::FrameResized(float width, float height)
{
	UpdateScrollBars();
	
	fLabel->ResizeTo(width,fLabel->Frame().Height());
	fLabel->MoveTo(Frame().left,fLabel->Frame().top);

	BListView::FrameResized(width - B_V_SCROLL_BAR_WIDTH,height);
}


bool FileSystemList::InitiateDrag(BPoint point,int32 index,bool wasSelected)
{
	return true;
}


/*************************************************************************************/
//  the filesystem item base class (currently not really necessary)
//	#pragma mark -


FileSystemItem::FileSystemItem()
{
}


/*************************************************************************************/
//  shows the filesystem in a list
//	#pragma mark -


LocalDirectoryItem::LocalDirectoryItem(char *volume, char *root, bool readOnly)
{
	fVolume = strdup(volume);
	fRoot = strdup(root);
	fReadOnly = readOnly;
}


LocalDirectoryItem::~LocalDirectoryItem()
{
	free(fVolume);
	free(fRoot);
}


void LocalDirectoryItem::DrawItem(BView *owner, BRect itemRect, bool drawEverything)
{
	owner->PushState();
	
	if (IsSelected())
		owner->SetLowColor(152,182,212);
	owner->FillRect(itemRect,B_SOLID_LOW);
	
	BPoint p(itemRect.left+6,itemRect.bottom-4);
	owner->SetHighColor(0,0,0);

	owner->SetDrawingMode(B_OP_ALPHA);
	owner->DrawBitmap(gLocalDirectoryBitmap,BPoint(itemRect.left + (kFileSystemTypeWidth - gLocalDirectoryBitmap->Bounds().Width())/2,
					  itemRect.top + (itemRect.Height() - gLocalDirectoryBitmap->Bounds().Height())/2));
	
	BString text = fVolume;  p.x += kFileSystemTypeWidth;
	owner->TruncateString(&text,B_TRUNCATE_END,kFileSystemVolumeWidth-6);	
	owner->DrawString(text.String(),p);
	
	text = fRoot;
	owner->TruncateString(&text,B_TRUNCATE_MIDDLE,itemRect.right - p.x - kFileSystemReadOnlyWidth - 6);
	p.x += kFileSystemVolumeWidth;
	owner->DrawString(text.String(),p);
	
	p.x = itemRect.right - kFileSystemReadOnlyWidth+6;
	owner->DrawString(fReadOnly ? "r/o" : "r/w",p);
	
	owner->PopState();
}


void LocalDirectoryItem::Update(BView *owner,const BFont *font)
{
	BListItem::Update(owner,font);

	font_height fontHeight;
	font->GetHeight(&fontHeight);
	
	float height = fontHeight.ascent + fontHeight.descent + fontHeight.leading + 4;

	if (Height() < height)
		SetHeight(height);
}


void LocalDirectoryItem::AddFileSystem(struct uaedev_mount_info *mountInfo)
{
	add_filesys_unit(mountInfo,fVolume,fRoot,fReadOnly ? 1 : 0,0,0,0,0);
}


/*************************************************************************************/
//  shows the hardfile in a list
//	#pragma mark -


HardFileItem::HardFileItem(char *file, bool readOnly, int sectorsPerTrack, int surfaces, int reservedBlocks, int blockSize)
{
	fFile = strdup(file);
	fReadOnly = readOnly;
	fSectorsPerTrack = sectorsPerTrack;
	fSurfaces = surfaces;
	fReservedBlocks = reservedBlocks;
	fBlockSize = blockSize;
}


HardFileItem::~HardFileItem()
{
	free(fFile);
}


void HardFileItem::DrawItem(BView *owner, BRect itemRect, bool drawEverything)
{
	owner->PushState();
	
	if (IsSelected())
		owner->SetLowColor(152,182,212);
	owner->FillRect(itemRect,B_SOLID_LOW);
	
	BPoint p(itemRect.left+6,itemRect.bottom-4);
	owner->SetHighColor(0,0,0);

	owner->SetDrawingMode(B_OP_ALPHA);
	owner->DrawBitmap(gHardFileBitmap,BPoint(itemRect.left + (kFileSystemTypeWidth - gHardFileBitmap->Bounds().Width())/2,
					  itemRect.top + (itemRect.Height() - gHardFileBitmap->Bounds().Height())/2));
	
	p.x += kFileSystemTypeWidth;
	owner->DrawString("-",p);
	
	BString text = fFile;
	owner->TruncateString(&text,B_TRUNCATE_MIDDLE,itemRect.right - p.x - kFileSystemReadOnlyWidth - 6);
	p.x += kFileSystemVolumeWidth;
	owner->DrawString(text.String(),p);
	
	p.x = itemRect.right - kFileSystemReadOnlyWidth+6;
	owner->DrawString(fReadOnly ? "r/o" : "r/w",p);
	
	owner->PopState();
}


void HardFileItem::Update(BView *owner,const BFont *font)
{
	BListItem::Update(owner,font);

	font_height fontHeight;
	font->GetHeight(&fontHeight);
	
	float height = fontHeight.ascent + fontHeight.descent + fontHeight.leading + 4;

	if (Height() < height)
		SetHeight(height);
}


void HardFileItem::AddFileSystem(struct uaedev_mount_info *mountInfo)
{
	add_filesys_unit(mountInfo,NULL,fFile,fReadOnly ? 1 : 0,fSectorsPerTrack,fSurfaces,fReservedBlocks,fBlockSize);
}

