#ifndef BE_FILESYSTEM_SETTINGS_H
#define BE_FILESYSTEM_SETTINGS_H
/***********************************************************/
//  UAE - The Un*x Amiga Emulator
//
//  BeOS port - filesystem gui settings
//
//  (c) 2001 Axel Doerfler
//
/***********************************************************/

#include <AppKit.h>
#include <InterfaceKit.h>
#include <KernelKit.h>
#include <FilePanel.h> 
#include <StorageKit.h>
#include <List.h>
#include <ListItem.h>
#include <String.h>

#include <MWindow.h>

class MView;
class MListView;
class MTextControl;
class MCheckBox;
class MPopup;
class MBorder;
class MButton;

class MemorySlider;
class FileSystemList;
class BetterScrollView;

const off_t kGBSize = 1024*1024*1024;
const off_t kMBSize = 1024*1024;
const off_t kKBSize = 1024;

/******************************** LocalDirectoryWindow ********************************/

class LocalDirectoryItem;

const uint32 kMsgFileSystemOk = 'fsok';
const uint32 kMsgFileSystemRoot = 'fsrt';
const uint32 kMsgFileSystemAccessMode = 'fsam';

class LocalDirectoryWindow : public MWindow
{
	public:
		LocalDirectoryWindow(BRect frame,BListView *list,LocalDirectoryItem *item);
		void MessageReceived(BMessage *msg);

	private:
		BListView *fListView;
		LocalDirectoryItem *fItem;
		BFilePanel *fFilePanel;
		
		MTextControl *fVolume,*fRoot;
		MPopup *fAccessMode;
		
		bool fReadOnly;
};

/******************************** HardFileWindow ********************************/

class HardFileItem;

const uint32 kMsgCreateHardFile = 'crhf';

class HardFileWindow : public MWindow
{
	public:
		HardFileWindow(BRect frame,BListView *list,HardFileItem *item);
		void MessageReceived(BMessage *msg);

	protected:
		status_t Alert(char *text,char *button1 = NULL,alert_type type = B_WARNING_ALERT);
		off_t ConvertSize(const char *text);
		void CreateHardFile();

		BListView *fListView;
		HardFileItem *fItem;
		BFilePanel *fFilePanel;
		
		MTextControl *fFile,*fSectorsPerTrack,*fSurfaces,*fReservedBlocks,*fBlockSize,*fSize;
		MPopup *fAccessMode;
		
		bool fReadOnly;
};

/******************************** FileSystemLabel ********************************/

class FileSystemLabel : public BView
{
	public:
		FileSystemLabel(BRect frame);
		virtual void Draw(BRect updateRect);

	protected:
		void DrawBox(BRect rect, char *label);
};

/******************************** FileSystemList ********************************/

class FileSystemList : public BListView
{
	public:
		FileSystemList(BRect frame);
		~FileSystemList();

		virtual void FrameResized(float width, float height);
		virtual bool InitiateDrag(BPoint point,int32 index,bool wasSelected);
		
		void UpdateScrollBars();
		inline MView *ScrollView() {return (MView *)fScrollView;};

	protected:
		FileSystemLabel *fLabel;
		BetterScrollView *fScrollView;
};

/******************************** FileSystemItem ********************************/

const int32 kIconLocalDirectory = 14;
const int32 kIconHardFile = 13;

class FileSystemItem : public BListItem
{
	public:
		FileSystemItem();

		virtual void AddFileSystem(struct uaedev_mount_info *mountInfo) = 0;
};

/******************************** LocalDirectoryItem ********************************/

class LocalDirectoryItem : public FileSystemItem
{
	public:
		LocalDirectoryItem(char *volume,char *root,bool readOnly);
		~LocalDirectoryItem();

		void DrawItem(BView *owner,BRect itemRect,bool drawEverything = false);
		void Update(BView *owner,const BFont *font);

		void AddFileSystem(struct uaedev_mount_info *mountInfo);

	private:
		friend class LocalDirectoryWindow;

		char *fVolume,*fRoot;
		bool fReadOnly;
};

/******************************** HardFileItem ********************************/

class HardFileItem : public FileSystemItem
{
	public:
		HardFileItem(char *file,bool readOnly,int sectorsPerTrack,int surfaces,int reservedBlocks,int blockSize);
		~HardFileItem();

		void DrawItem(BView *owner,BRect itemRect,bool drawEverything = false);
		void Update(BView *owner,const BFont *font);

		void AddFileSystem(struct uaedev_mount_info *mountInfo);

	private:
		friend class HardFileWindow;

		char *fFile;
		int  fSectorsPerTrack,fSurfaces,fReservedBlocks,fBlockSize;
		bool fReadOnly;
};

#endif
