/***********************************************************/
//  BeUAE - The Be Un*x Amiga Emulator
//
//  BeOS port keyboard/joystick routines
//
//  (c) 2000-2001 Axel Drfler
//  (c) 1999 Be/R4 Sound - Raphael Moll
//  (c) 1998-1999 David Sowsy
//  (c) 1996-1998 Christian Bauer
//  (c) 1996 Patrick Hanevold
//
/***********************************************************/

#include <Joystick.h>

#include "be-UAE.h"
#include "be-Window.h"
#include "be-Settings.h"
#include "be-Input.h"

extern "C" {
#include "sysconfig.h"
#include "sysdeps.h"
#include "config.h"
#include "options.h"
#include "xwin.h"
#include "joystick.h"
#include "keybuf.h"
#include "gui.h"
#include "uae.h"
};

static BJoystick *joystick;
int nr_joysticks;
static key_info lastKeyInfo;
static bool lastKeyInfoInitialized = false;

// Speed control hacks by David Sowsy
int mouse_speed_rate = 1;

#define key_pressed(k) (keyInfo.key_states[k >> 3] & (1 << (~k & 7)))
const int RIGHT_ALT_KEY = 0x5f;
const int LEFT_ALT_KEY = 0x5d;
const int LEFT_CONTROL_KEY = 0x5c;
const int RIGHT_CONTROL_KEY = 0x60;

//const int NUMPAD_ENTER = 0x5b;
//const int NUMPAD_1_END = 0x58;
//const int NUMPAD_2_DOWN = 0x59;
//const int NUMPAD_3_PAGE_DOWN = 0x5a;
//const int NUMPAD_4_LEFT = 0x48;
//const int NUMPAD_5 = 0x49;
//const int NUMPAD_6_RIGHT = 0x4a;
//const int NUMPAD_7_POS1 = 0x37;
//const int NUMPAD_8_UP = 0x38;
//const int NUMPAD_9_PAGE_UP = 0x39;


//  Poll mouse and keyboard
void handle_events(void)
{
	int be_code,be_byte,be_bit,amiga_code;
	key_info keyInfo;

	if (!lastKeyInfoInitialized)
	{
		get_key_info(&lastKeyInfo);
		lastKeyInfoInitialized = true;
	}

	// Redraw drive LEDs
	/*for (int i=0; i<4; i++)
		DriveLED[i]->SetState(LEDs[i]);*/

	if (gEmulationWindow->UpdateMouseButtons())
	{
		get_key_info(&keyInfo);

		// Keyboard
		if (memcmp(keyInfo.key_states, lastKeyInfo.key_states, sizeof(keyInfo.key_states)))
		{
			for(be_code = 0;be_code < 0x80;be_code++)
			{
				be_byte = be_code >> 3;
				be_bit = 1 << (~be_code & 7);

				// Key state changed?
				if (	(keyInfo.key_states[be_byte] & be_bit)
					!= 	(lastKeyInfo.key_states[be_byte] & be_bit))
				{
					amiga_code = keycode2amiga[be_code];
					if (keyInfo.key_states[be_byte] & be_bit)
					{
						// Key pressed
						if (amiga_code == AK_mousestuff)
							togglemouse();
						else
							record_key(amiga_code << 1);
					}
					else	// Key released
						record_key((amiga_code << 1) | 1);
				}
			}		
			lastKeyInfo = keyInfo;
		}

		if (key_pressed(B_F11_KEY) || key_pressed(B_F12_KEY))
		{
			bool fullScreen = false;
			
			if (gEmulationWindow->IsFullScreen())
			{
				gEmulationWindow->SetFullScreenMode(false);
				fullScreen = true;
			}
			openGUI(true);
			
			if (fullScreen)
				gEmulationWindow->SetFullScreenMode(true);
		}
		// "Affengriff"
		if (	key_pressed(LEFT_ALT_KEY)
			&&	key_pressed(RIGHT_ALT_KEY)
			&&	key_pressed(LEFT_CONTROL_KEY))
			uae_reset();

		if (inhibit_frame != 2) 
			inhibit_frame = keyInfo.key_states[0x0f >> 3] &( 1 << (~0x0f & 7));
	}
}


/*************************************************************************************/
// Joystick routines
// #pragma mark -


void read_joystick(int nr, unsigned int *dir, int *button)
{
	static int joy_minx = 32767, joy_maxx = 0,
			   joy_miny = 32767, joy_maxy = 0;

	*dir = 0;
	*button = 0;

	if (nr >= nr_joysticks)
		return;

	if (joystick->Update() != B_OK)
		return;

	if (joystick->horizontal > joy_maxx)
		joy_maxx = joystick->horizontal;
	if (joystick->horizontal < joy_minx)
		joy_minx = joystick->horizontal;
	if (joystick->vertical > joy_maxy)
		joy_maxy = joystick->vertical;
	if (joystick->vertical < joy_miny)
		joy_miny = joystick->vertical;

	if (joy_maxx-joy_minx < 100 || joy_maxy-joy_miny < 100)
		return;

	int left = 0, right = 0, top = 0, bottom = 0;
	
	if (joystick->horizontal < (joy_minx + (joy_maxx-joy_minx)/3))
		right = 1;
	else if (joystick->horizontal > (joy_minx + 2*(joy_maxx-joy_minx)/3))
		left = 1;

	if (joystick->vertical < (joy_miny + (joy_maxy-joy_miny)/3))
		bottom = 1;
	else if (joystick->vertical > (joy_miny + 2*(joy_maxy-joy_miny)/3))
		top = 1;

	if (left) top = !top;
	if (right) bottom = !bottom;
    *dir = bottom | (right << 1) | (top << 8) | (left << 9);
    *button = !joystick->button1;
}


void init_joystick(void)
{
	joystick = new BJoystick();

	char deviceName[B_OS_NAME_LENGTH];
	if (joystick->GetDeviceName(0,deviceName) == B_OK)
	{
		printf("Joystick device = \"%s\"\n",deviceName);	// e.g. "es1371/1"
		
		if (joystick->Open(deviceName) != B_ERROR) 
			nr_joysticks = 1;
	}
}


void close_joystick(void)
{
	joystick->Close();
	delete joystick;
}

