/***********************************************************/
//  BeUAE - The Be Un*x Amiga Emulator
//
//  BeOS port - gui settings
//
//  (c) 2000,2001 Axel Drfler
//
/***********************************************************/

#include <Application.h>
#include <Window.h>
#include <Resources.h>
#include <experimental/ResourceSet.h>

#include <ctype.h>
#include <string.h>

#include "be-UAE.h"
#include "be-Window.h"
#include "be-Settings.h"
#include "be-Input.h"
#include "be-FileSystemSettings.h"

#include <VGroup.h>
#include <HGroup.h>
#include <LayeredGroup.h>
#include <MBorder.h>
#include <MScrollView.h>
#include <MListView.h>
#include <MTextView.h>
#include <MTextControl.h>
#include <MStringView.h>
#include <MCheckBox.h>
#include <MRadioGroup.h>
#include <MPopup.h>
#include <MButton.h>
#include <MSlider.h>
#include <Space.h>
#include <MMenuBar.h>
#include <MBViewWrapper.h>
#include "santa/BetterScrollView.h"

extern "C" {
#include "sysconfig.h"
#include "sysdeps.h"
#include "config.h"
#include "options.h"
#include "gui.h"
#include "xwin.h"
#include "custom.h"
#include "picasso96.h"
#include "disk.h"
#include "autoconf.h"
#include "uae.h"
};

// settings constants
const uint32 kMinChipRAM = 0x80000, kMaxChipRAM = 0x200000;		// 512kb - 2MB
const uint32 kMinFastRAM = 0x100000, kMaxFastRAM = 0x800000;	// 1MB - 8MB
const uint32 kMinSlowRAM = 0x80000, kMaxSlowRAM = 0x100000;		// 512kb - 1MB
const uint32 kMinZ3RAM = 0x100000, kMaxZ3RAM = 0x4000000;		// 1MB - 64MB
const uint32 kMinGfxRAM = 0x100000, kMaxGfxRAM = 0x800000;		// 1MB - 8MB

// external variables
extern const char *kConfigurationMIMEType;
extern BString gSettingsName;
extern display_mode gDisplayMode;
extern BSoundPlayer *gSoundPlayer;

// global variables
struct be_prefs gBePrefs;
UAESettings *gSettingsWindow;
BMessenger *gSettingsMessenger;
BBitmap *gLocalDirectoryBitmap,*gHardFileBitmap;
BEntry	*gSettingsEntry;
sem_id	gSettingsLock;
bool	gQuit;


UAESettings::UAESettings(BRect frame,bool isRunning)
	: MWindow(frame,"BeUAE Settings",B_TITLED_WINDOW,0),
	fIsRunning(isRunning)
{
	acquire_sem(gSettingsLock);
	gSettingsMessenger = new BMessenger(this);
	gQuit = true;

	fListView = new MListView(B_SINGLE_SELECTION_LIST,minimax(100,280,1E5,1E5,1));
	
	fListView->AddItem(new IconItem("CPU",kIconRAM,kIconSelectedRAM));
	fListView->AddItem(new IconItem("ROM/RAM",kIconRAM,kIconSelectedRAM));
	fListView->AddItem(new IconItem("Video",kIconVideo,kIconSelectedVideo));
	fListView->AddItem(new IconItem("Audio",kIconAudio,kIconSelectedAudio));
	fListView->AddItem(new IconItem("Input",kIconInput,kIconSelectedInput));
	fListView->AddItem(new IconItem("Floppy Disk",kIconFloppydisk,kIconSelectedFloppydisk));
	fListView->AddItem(new IconItem("Hard Disk",kIconHarddisk,kIconSelectedHarddisk));
	fListView->SetSelectionMessage(new BMessage(kMsgSettingsPage));

	if (isRunning)
	{
		int count = fListView->CountItems();
		
		for(int i = 0;i < count;i++)
			if (i != 5)
				fListView->ItemAt(i)->SetEnabled(false);

		fPrefs = &changed_prefs;
	}
	else
		fPrefs = &currprefs;

	MMenuBar *menuBar;
	HGroup *hGroup;
	VGroup *vGroup;
	MButton *button;
	MTextView *textView;

	fFileSystemView = new FileSystemList(BRect(0,0,200,200));

	MView *topView = new VGroup
	(
		menuBar = new MMenuBar(),
		new HGroup
		(
			new MScrollView(fListView,false,true,B_FANCY_BORDER,minimax(100,280,1E5,1E5,1)),
			new Space(kHorizontal),
			fPagesView = new LayeredGroup
			(
				minimax(420,-1,1E5,1E5,5),
				
				/************ CPU ************/

				new VGroup
				(
					new Space,
					new MBorder
					(
						M_LABELED_BORDER,kBorderSpacing,"CPU Type/Model",
						new VGroup
						(
							new HGroup
							(
								fCPU = new MPopup("Model:",new BMessage(kMsgCPU),NULL,
									"68000","68010","68020","68040",NULL),
								fEnableFPU = new MCheckBox("Enable FPU"),
								f24BitAddressMode = new MCheckBox("24bit Address Mode"),
								NULL
							),
							new Space(minimax(0,10,0,10)),
							fCPUCompatible = new MCheckBox("Be as compatible as possible (but slower)"),
							NULL
						)
					),
					new MBorder
					(
						M_LABELED_BORDER,kBorderSpacing,"CPU Speed Settings",
						new HGroup
						(
							fCPUSpeed = new MPopup("Speed:",new BMessage(kMsgCPUSpeed),NULL,
								"Maximum","Approximate A500","Adjust between CPU and Custom Chips",NULL),
							fCPURate = new SpeedRateSlider(1,10240),
							NULL
						)
					),
					new Space,
					NULL
				),
				
				/************ ROM/RAM ************/

				new VGroup
				(
					new Space,
					new MBorder
					(
						M_LABELED_BORDER,kBorderSpacing,"ROM",
						new VGroup
						(
							createFileView(&fROMFile,"ROM/Kickstart File:",kMsgROMFile),
							createFileView(&fROMKey, "ROM/Kickstart Key File:",kMsgROMKey),
//							new Space(kVertical),
//							fROMAutoConfig = new MCheckBox("Kickstart supports AutoConfig™ (Kickstart 1.3 and higher)"),
							NULL
						)
					),
					new Space(kVertical),
					new HGroup
					(
						new MBorder(M_LABELED_BORDER,kSliderSpacing,"Chip-RAM",
									fChipMemory = new MemorySlider(kMinChipRAM,kMaxChipRAM,false)),
						new MBorder(M_LABELED_BORDER,kSliderSpacing,"Slow/Bogo-RAM",
									fSlowMemory = new MemorySlider(kMinSlowRAM,kMaxSlowRAM,true)),
						NULL
					),
					new Space(kVertical),
					new MBorder(M_LABELED_BORDER,kSliderSpacing,"Fast-RAM",
								fFastMemory = new MemorySlider(kMinFastRAM,kMaxFastRAM,true)),
					new Space(kVertical),
					new MBorder(M_LABELED_BORDER,kSliderSpacing,"Z3-RAM",
								fZ3Memory = new MemorySlider(kMinZ3RAM,kMaxZ3RAM,true)),
					new Space,
					NULL
				),
				
				/************ Video ************/

				new VGroup
				(
					new Space,
					new HGroup
					(
						new MBorder
						(
							M_LABELED_BORDER,kBorderSpacing,"Amiga",
							new VGroup
							(
								fGfxSize = new MPopup("Window Size:",new BMessage(kMsgGfxSize),NULL,
									"320 x 256","640 x 480","800 x 600","1024 x 768","1280 x 1024",NULL),
								fGfxDepth = new MPopup("Screen Depth:",new BMessage(kMsgGfxDepth),NULL,
									"8bit","15bit","16bit","32bit","Adapt to Workspace",NULL),
								fGfxFullScreen = new MCheckBox("Use Full-Screen Mode",0,true),
								fGfxCorrectAspect = new MCheckBox("Correct Aspect",0,true),
								fGfxResize = new MCheckBox("Resize view dynamically",0,true),
								new Space(kVertical),
								fGfxFrameRate = new FrameRateSlider(1,10),
								NULL
							)
						),
						new VGroup
						(
							new MBorder
							(
								M_LABELED_BORDER,kBorderSpacing,"Chipset",
								new VGroup
								(
									fGfxChipset = new MPopup("Chipset:",new BMessage(kMsgGfxChipset),NULL,
										"OCS","ECS Agnus","ECS Denise","ECS","AGA",NULL),
									fGfxImmediateBlits = new MCheckBox("Immediate Blit"),
									fGfx32BitBlits = new MCheckBox("Enable 32bit blits"),
									fGfxNTSC = new MCheckBox("NTSC video mode"),
									NULL
								)
							),
							new MBorder
							(
								M_LABELED_BORDER,kBorderSpacing,"Centering",
								new VGroup
								(
									fGfxCenterHorizontal = new MCheckBox("Center Horizontal"),
									fGfxCenterVertical = new MCheckBox("Center Vertical"),
									NULL
								)
							),
							NULL
						),
						NULL
					),
					new Space(kVertical),
					new MBorder
					(
						M_LABELED_BORDER,kBorderSpacing,"Picasso96",
						new VGroup
						(
							fPicassoFullScreen = new MCheckBox("Use Full-Screen Mode",0,true),
							new Space(kVertical),
							new MBorder(M_LABELED_BORDER,kSliderSpacing,"Graphics Card-RAM",
										fGfxMemory = new MemorySlider(kMinGfxRAM,kMaxGfxRAM,true)),
							NULL
						)
					),
					new Space,
					NULL
				),
				
				/************ Audio ************/

				new HGroup
				(
					new Space,
					new VGroup
					(
						new Space,
						fSoundMode = new MPopup("Sound mode:",new BMessage(kMsgSoundMode),NULL,
							"Disabled",
							"Disabled but Emulated",
							"Standard Sound",
							"Exact Sound",
							NULL
						),
						new Space(kVertical),
						new MBorder
						(
							M_LABELED_BORDER,kBorderSpacing,"Sound settings",
							new VGroup
							(
								new HGroup
								(
									new VGroup
									(
										fSoundFrequency = new MPopup("Frequency:",new BMessage(kMsgSoundFrequency),NULL,
											"11025","22050","44100",NULL),
										fSoundBits = new MPopup("Resolution:",new BMessage(kMsgSoundBits),NULL,
											"8 bits","16 bits",NULL),
										NULL
									),
									new VGroup
									(
										fSoundChannels = new MPopup("Channels:",new BMessage(kMsgSoundChannels),NULL,
											"mono","stereo","mixed",NULL),
										fSoundInterpolation = new MPopup("Interpolation:",new BMessage(kMsgSoundInterpolation),NULL,
											"none","rh","crux",NULL),
										NULL
									),
									NULL
								),
								new Space(kVertical),
								new MBorder(M_LABELED_BORDER,kSliderSpacing,"Sound Buffer",
										fSoundBuffer = new MemorySlider(256,8192,false)),
								NULL
							)
						),
						new Space(kVertical),
						textView = new MTextView(minimax(300,80,1e6,1e6)),
						new Space,
						NULL
					),
					new Space,
					NULL
				),
				
				/************ Input ************/

				new VGroup
				(
					new Space,
					new HGroup
					(
						new MBorder(M_LABELED_BORDER,kBorderSpacing,"Joystick Port 1",
							fJoystickPort0 = new MRadioGroup(new BMessage(kMsgJoystickPort0),"Joystick 1","Joystick 2","Mouse","Keyboard A: Numeric Pad","Keyboard B: Cursor Keys","Keyboard C",NULL)),
						new MBorder(M_LABELED_BORDER,kBorderSpacing,"Joystick Port 2",
							fJoystickPort1 = new MRadioGroup(new BMessage(kMsgJoystickPort1),"Joystick 1","Joystick 2","Mouse","Keyboard A: Numeric Pad","Keyboard B: Cursor Keys","Keyboard C",NULL)),
						NULL
					),
					fKeyboardLanguage = new MPopup("Keyboard language:",new BMessage(kMsgKeyboardLanguage),NULL,
						"US/English","Deutsch","Svenska","Français","Italiano","Español",NULL),
					new Space,
					NULL
				),
								
				/************ Floppy Disk ************/

				new VGroup
				(
					new Space(kVertical),
					CreateFloppyView(0),
					new Space(kVertical),
					CreateFloppyView(1),
					new Space(kVertical),
					CreateFloppyView(2),
					new Space(kVertical),
					CreateFloppyView(3),
					new Space,
					new MButton("Create blank floppy disk file",new BMessage(kMsgCreateFloppyFile)),
					new Space,
					NULL
				),
				
				/************ Hard Disk ************/

				//new MStringView("Hard Disk options not yet available"),
				new VGroup
				(
					new Space(kVertical),
					new MBorder(M_LABELED_BORDER,kBorderSpacing,"Mounted Local Directories & Hardfiles",
						new VGroup
						(
							fFileSystemView->ScrollView(),
							new HGroup
							(
								new MButton("Add local dir" B_UTF8_ELLIPSIS,new BMessage(kMsgAddLocalDirectory)),
								new MButton("Add hardfile" B_UTF8_ELLIPSIS,new BMessage(kMsgAddHardFile)),
								fEditFileSystem = new MButton("Edit" B_UTF8_ELLIPSIS,new BMessage(kMsgEditFileSystem)),
								fRemoveFileSystem = new MButton("Remove",new BMessage(kMsgRemoveFileSystem)),
								NULL
							),
							new HGroup
							(
								new Space(),
								new MStringView("List order sets boot priority"),
								fUpFileSystem = new MButton("Up",new BMessage(kMsgUpFileSystem)),
								fDownFileSystem = new MButton("Down",new BMessage(kMsgDownFileSystem)),
								NULL
							),
							NULL
						)),
					new Space(kVertical),
					NULL
				),
				NULL
			),
			new Space(kHorizontal),
			NULL
		),
		vGroup = new VGroup
		(
			new Space(kVertical),
				new HGroup
				(
					new MButton(isRunning ? "Continue" : "Start UAE",new BMessage(kMsgStart)),
					isRunning ? new MButton("Reset",new BMessage(kMsgReset))
							  : new MButton("Quit",new BMessage(kMsgQuit)),
					isRunning ? new MButton("Quit",new BMessage(kMsgQuit)) : NULL,
					NULL
				),
			NULL
		),
		NULL
	);
	menuBar->flags |= M_NO_Y_LEFTOVERS;
	vGroup->flags |= M_NO_Y_LEFTOVERS;

	DivideSame(fSoundFrequency,fSoundBits,NULL);
	DivideSame(fSoundChannels,fSoundInterpolation,NULL);

	// picasso supports only full screen for now
	fPicassoFullScreen->SetEnabled(false);

	// audio warning/note
	textView->SetText("Disabling the sound is equivalent with disabling the timing in UAE.\n"
					  "If you are encountering any problems with timing, try to alter the size of the sound buffer.\n"),
	textView->MakeEditable(false);
	textView->MakeSelectable(false);

	// set filesys messages
	fFileSystemView->SetSelectionMessage(new BMessage(kMsgFileSystemList));
	fFileSystemView->SetInvocationMessage(new BMessage(kMsgEditFileSystem));

	gLocalDirectoryBitmap = retrieveBitmap(kIconLocalDirectory);
	gHardFileBitmap = retrieveBitmap(kIconHardFile);

	// disable edit/remove filesys gadgets
	fEditFileSystem->SetEnabled(false);
	fRemoveFileSystem->SetEnabled(false);
	fUpFileSystem->SetEnabled(false);
	fDownFileSystem->SetEnabled(false);

	/*** make menu ***/
	
	BMenuItem *item;
	
	BMenu *menu = new BMenu("File");
	menu->AddItem(new BMenuItem("New",new BMessage(kMsgMenuNew),'N'));
	menu->AddItem(item = new BMenuItem("Open" B_UTF8_ELLIPSIS,new BMessage(kMsgMenuOpen),'O'));
//	item->SetTarget(be_app);
	menu->AddSeparatorItem();
	menu->AddItem(fSaveItem = new BMenuItem("Save",new BMessage(kMsgMenuSave),'S'));
	fSaveItem->SetEnabled(gSettingsEntry != NULL);
	menu->AddItem(new BMenuItem("Save as" B_UTF8_ELLIPSIS,new BMessage(kMsgMenuSaveAs)));
	menu->AddSeparatorItem();
	menu->AddItem(item = new BMenuItem("About" B_UTF8_ELLIPSIS,new BMessage(B_ABOUT_REQUESTED),'?'));
//	item->SetTarget(be_app);
	menu->AddItem(new BMenuItem("Quit",new BMessage(kMsgQuit),'Q'));
	menuBar->AddItem(menu);

	AddChild(dynamic_cast<BView*>(topView));

	// Create file panels (open and save)
	fFilePanel = new BFilePanel(B_OPEN_PANEL,gSettingsMessenger,NULL,0,false);
	fFilePanelMode = kFilePanelModeNone;
	fFloppyPanelUnit = 0;
	fSaveFilePanel = new BFilePanel(B_SAVE_PANEL,gSettingsMessenger,NULL,0,false);
	
	UpdateViews();
	AddCommonFilter(new BMessageFilter(B_SIMPLE_DATA,UAESettings::MessageDropFilter));
	
	// a MWindow derived class would do this automatically, but LayeredGroup::ActivateLayer()
	// requires this call
	topView->layoutprefs();

	fPagesView->ActivateLayer(isRunning ? 5 : 0);
	fListView->Select(isRunning ? 5 : 0);
	
}


UAESettings::~UAESettings()
{
	for(int i = 0;i < 4;i++)
		free(fFloppyLabel[i]);

	delete fFilePanel;
	delete fSaveFilePanel;
	delete gSettingsMessenger;
	
	gSettingsMessenger = NULL;
}


MView *UAESettings::CreateFloppyView(int unit)
{
	uint32 selectID,ejectID;
	char label[32];
	
	sprintf(label,"Floppy Unit %d",unit);
	selectID = 'sDF0' + unit;
	ejectID = 'eDF0' + unit;
	fFloppyLabel[unit] = strdup(label);

	VGroup *vGroup;
	MBorder *border = new MBorder
	(
		M_LABELED_BORDER,kBorderSpacing,fFloppyLabel[unit],
		vGroup = new VGroup
		(
			fFloppyPath[unit] = new MTextControl("Disk File:",fPrefs->df[unit]),
			new HGroup
			(
				new Space,
				new MButton("Select" B_UTF8_ELLIPSIS,new BMessage(selectID)),
				new MButton("Eject",new BMessage(ejectID)),
				NULL
			),
			//new Space(minimax(0,6,0,6,1)),
			NULL
		)
	);
	vGroup->flags |= M_NO_Y_LEFTOVERS;
	return border;
}


bool UAESettings::QuitRequested()
{
	UpdateConfig();

	release_sem(gSettingsLock);

	// notify the main application if UAE is going down
	if (gQuit)
		be_app->PostMessage(B_QUIT_REQUESTED);

	return true;
}


void UAESettings::SetEnabledRadioButton(MRadioGroup *radioGroup,int inactive)
{
	int count = radioGroup->CountChildren();
	BControl *control;
			
	for(int i = 0;i < count;i++)
		if ((control = dynamic_cast<BControl *>(radioGroup->ChildAt(i))) != NULL)
			control->SetEnabled(i != inactive);
}


void UAESettings::UpdateConfig()
{
	// memory sizes
	fPrefs->fastmem_size = fFastMemory->MemoryValue();
	fPrefs->chipmem_size = fChipMemory->MemoryValue();
	fPrefs->z3fastmem_size = fZ3Memory->MemoryValue();
	fPrefs->bogomem_size = fSlowMemory->MemoryValue();
	fPrefs->gfxmem_size = fGfxMemory->MemoryValue();

	// cpu
	if (fPrefs->cpu_level == 2 && fEnableFPU->Value())
		fPrefs->cpu_level = 3;	// 68020 with FPU
	fPrefs->address_space_24 = f24BitAddressMode->Value();
	fPrefs->cpu_compatible = fCPUCompatible->Value();
	if (fPrefs->m68k_speed > 0)
		fPrefs->m68k_speed = fCPURate->Value();

	// sound settings
	fPrefs->sound_minbsiz = fPrefs->sound_maxbsiz = fSoundBuffer->MemoryValue();

	// rom
	strcpy(fPrefs->romfile,fROMFile->Text());
	strcpy(fPrefs->keyfile,fROMKey->Text());
//	fPrefs->automount_uaedev = fROMAutoConfig->Value();
	
	// gfx
	fPrefs->gfx_correct_aspect = fGfxCorrectAspect->Value();
	fPrefs->gfx_afullscreen = fGfxFullScreen->Value();
	fPrefs->gfx_framerate = fGfxFrameRate->Value();
	fPrefs->immediate_blits = fGfxImmediateBlits->Value();
	fPrefs->blits_32bit_enabled = fGfx32BitBlits->Value();
	fPrefs->ntscmode = fGfxNTSC->Value();
	fPrefs->gfx_xcenter = fGfxCenterHorizontal->Value() ? 2 : 0;
	fPrefs->gfx_ycenter = fGfxCenterVertical->Value() ? 2 : 0;
	gBePrefs.dynamic_resize = fGfxResize->Value();
	
	// input
	fPrefs->jport0 = fJoystickPort0->ActiveButton();
	fPrefs->jport1 = fJoystickPort1->ActiveButton();
	
	// floppy disks
	for(int i = 0;i < 4;i++)
	{
		disk_eject(i);
		disk_insert(i,fFloppyPath[i]->Text());
	}

	// hard disks
	free_mountinfo(fPrefs->mountinfo);
	fPrefs->mountinfo = alloc_mountinfo();

	if (fPrefs->mountinfo != NULL)
	{
		FileSystemItem *fsItem;
		int count = fFileSystemView->CountItems();

		for(int i = 0;i < count;i++)
		{
			if ((fsItem = dynamic_cast<FileSystemItem *>(fFileSystemView->ItemAt(i))) != NULL)
				fsItem->AddFileSystem(fPrefs->mountinfo);
		}
	}
}


void UAESettings::UpdateViews()
{
	// memory sizes
	fFastMemory->SetMemoryValue(fPrefs->fastmem_size);
	fChipMemory->SetMemoryValue(fPrefs->chipmem_size);
	fZ3Memory->SetMemoryValue(fPrefs->z3fastmem_size);
	fSlowMemory->SetMemoryValue(fPrefs->bogomem_size);
	fGfxMemory->SetMemoryValue(fPrefs->gfxmem_size);
	
	// floppy units
	for(int i = 0;i < 4;i++)
		fFloppyPath[i]->SetText(fPrefs->df[i]);

	// cpu
	if (fPrefs->cpu_level < 2)	// 68000 & 68010
		fCPU->SetActive(fPrefs->cpu_level);
	else if (fPrefs->cpu_level < 4)	// 68020
		fCPU->SetActive(2);
	else	// 68040
		fCPU->SetActive(3);

printf("m68k Speed = %ld (%ld)\n",fPrefs->m68k_speed,min(fPrefs->m68k_speed + 1,2));
	fCPUSpeed->SetActive(min(fPrefs->m68k_speed + 1,2));
	fCPUCompatible->SetValue(fPrefs->cpu_compatible);
		
	// rom/key file, AutoConfig
	fROMFile->SetText(fPrefs->romfile);
	fROMKey->SetText(fPrefs->keyfile);
//	fROMAutoConfig->SetValue(fPrefs->automount_uaedev);

	// gfx
	{
		int value;
		
		if (fPrefs->gfx_width <= 320)
			value = 0;
		else if (fPrefs->gfx_width <= 640)
			value = 1;
		else if (fPrefs->gfx_width <= 800)
			value = 2;
		else if (fPrefs->gfx_width <= 1024)
			value = 3;
		else
			value = 4;
		fGfxSize->SetActive(value);
	
		if (fPrefs->color_mode == 0 || fPrefs->color_mode == 3 || fPrefs->color_mode == 4)
			value = 0;
		else if (fPrefs->color_mode == 1)
			value = 1;
		else if (fPrefs->color_mode == 2)
			value = 2;
		else if (fPrefs->color_mode == 5)
			value = 3;
		else
			value = 4;
		fGfxDepth->SetActive(value);
		fGfxResize->SetValue(gBePrefs.dynamic_resize);

		if (fPrefs->chipset_mask == 0)
			value = 0;
		else if (fPrefs->chipset_mask & CSMASK_AGA)
			value = 4;
		else if (fPrefs->chipset_mask & CSMASK_ECS_AGNUS)
		{
			if (fPrefs->chipset_mask & CSMASK_ECS_DENISE)
				value = 3;
			else
				value = 1;
		}
		else if (fPrefs->chipset_mask & CSMASK_ECS_DENISE)
			value = 2;
		fGfxChipset->SetActive(value,false);
	}
	fGfxCorrectAspect->SetValue(fPrefs->gfx_correct_aspect);
	fGfxFullScreen->SetValue(fPrefs->gfx_afullscreen);
	fGfxFrameRate->SetValue(fPrefs->gfx_framerate);
	fGfxImmediateBlits->SetValue(fPrefs->immediate_blits);
	fGfx32BitBlits->SetValue(fPrefs->blits_32bit_enabled);
	fGfxNTSC->SetValue(fPrefs->ntscmode);
	fGfxCenterHorizontal->SetValue(fPrefs->gfx_xcenter);
	fGfxCenterVertical->SetValue(fPrefs->gfx_ycenter);
		
	// picasso96
	fPicassoFullScreen->SetValue(1 /*fPrefs->gfx_pfullscreen*/);
		// picasso supports only full screen for now

	// sound
	fSoundMode->SetActive(fPrefs->produce_sound);
	fSoundFrequency->SetActive((fPrefs->sound_freq/11025) >> 1,false);
	fSoundChannels->SetActive(fPrefs->stereo,false);
	fSoundBits->SetActive(fPrefs->sound_bits/8 - 1,false);
	fSoundInterpolation->SetActive(fPrefs->sound_interpol,false);
	fSoundInterpolation->SetEnabled(fPrefs->sound_bits == 16);
	fSoundBuffer->SetMemoryValue(fPrefs->sound_minbsiz);
	
	// input
	fJoystickPort0->SetActive(fPrefs->jport0);
	SetEnabledRadioButton(fJoystickPort1,fPrefs->jport0);
	fJoystickPort1->SetActive(fPrefs->jport1);
	SetEnabledRadioButton(fJoystickPort0,fPrefs->jport1);
	fKeyboardLanguage->SetActive(fPrefs->keyboard_lang);
	
	// floppy disks
	
	// harddrives
	
	int count = fFileSystemView->CountItems();
	for(int i = 0;i < count;i++)
	{
		FileSystemItem *item = dynamic_cast<FileSystemItem *>(fFileSystemView->ItemAt(0));
		
		fFileSystemView->RemoveItem((int32)0);
		delete item;
	}
	char *volume,*root;
	int secs,surfaces,blockSize,readOnly,reserved,size,cylinders;
	
	for(int i = 0;get_filesys_unit(fPrefs->mountinfo,i,&volume,&root,&readOnly,&secs,&surfaces,&reserved,&cylinders,&size,&blockSize) == NULL;i++)
	{
		if (volume == NULL)
			fFileSystemView->AddItem(new HardFileItem(root,readOnly,secs,surfaces,reserved,blockSize));
		else
			fFileSystemView->AddItem(new LocalDirectoryItem(volume,root,readOnly));
	}
}


void UAESettings::OpenSaveFilePanel(int32 panelMode,char *title)
{
	fSaveFilePanelMode = panelMode;

	fSaveFilePanel->Hide();
	fSaveFilePanel->Window()->SetTitle(title);
	fSaveFilePanel->Show();
}


void UAESettings::SaveRequested(BMessage *msg)
{
	entry_ref ref;
	
	if (msg->FindRef("directory", &ref) == B_NO_ERROR)
	{
		BDirectory directory(&ref);
		if (directory.InitCheck() == B_NO_ERROR)
		{
			BString name;
					
			if (msg->FindString("name",&name) == B_NO_ERROR)
			{
				switch(fSaveFilePanelMode)
				{
					case kFilePanelModeFloppy:
					{
						BFile file;
						if (directory.CreateFile(name.String(),&file,true) == B_OK)
						{
							if (CreateFloppyFile(&file) == B_OK)
								break;
						}
						(new BAlert("","Could not create floppy disk file","Ok",NULL,NULL,B_WIDTH_FROM_LABEL,B_WARNING_ALERT))->Go(NULL);		
						break;
					}
					case kFilePanelModeConfig:
						BPath path(&directory,name.String());

						// update fPrefs to the current state
						UpdateConfig();
						
						// save current preferences data
						cfgfile_save(fPrefs,path.Path());
						
						// set the file type
						BNode node(path.Path());
						if (node.InitCheck() == B_NO_ERROR)
						{
							BNodeInfo nodeInfo(&node);
							if (nodeInfo.InitCheck() == B_NO_ERROR)
								nodeInfo.SetType(kConfigurationMIMEType);
						}
						break;
				}
			}
		}
	}
}


void UAESettings::OpenFilePanel(MTextControl *control,int32 panelMode,char *title)
{
	fFilePanelMode = panelMode;

	fFilePanel->Hide();
	fFilePanel->Window()->SetTitle(title);

	fFilePanelControl = control;
	
	char *text = ".";
	
	if (control != NULL)
		text = (char *)control->Text();
//	else if (panelMode == kFilePanelModeConfig)
//		text = ".";

	BPath path(text);
	if (path.InitCheck() == B_NO_ERROR)
		if (path.GetParent(&path) == B_NO_ERROR)
			fFilePanel->SetPanelDirectory(path.Path());

	fFilePanel->Show();
}


void UAESettings::RefsReceived(BMessage *msg)
{
	if (fFilePanelMode == kFilePanelModeNone)
		return;

	entry_ref ref;
	
	if (msg->FindRef("refs", &ref) == B_NO_ERROR)
	{
		BEntry entry(&ref);
		if (entry.InitCheck() == B_NO_ERROR)
		{
			BPath path;
			entry.GetPath(&path);

			if (fFilePanelControl != NULL)
				fFilePanelControl->SetText(path.Path());
			
			switch(fFilePanelMode)
			{
//				case kFilePanelModeFloppy:
//					break;
				case kFilePanelModeConfig:
					default_prefs(fPrefs);
					cfgfile_load(fPrefs,path.Path());
					
					delete gSettingsEntry;
					gSettingsEntry = new BEntry(entry);
					fSaveItem->SetEnabled(true);

					UpdateViews();
					break;
			}
		}
	}
}


filter_result UAESettings::MessageDropFilter(BMessage *message,BHandler **,BMessageFilter *filter)
{
	if (message->WasDropped())
		dynamic_cast<UAESettings *>(filter->Looper())->RefsReceived(message);
	
	return B_SKIP_MESSAGE;	
}


void UAESettings::MessageReceived(BMessage *msg)
{
	switch(msg->what)
	{
		/******** CPU ********/

		case kMsgCPU:
		{
			int32 cpu = popUpIndex(msg);
			
			if (cpu < 2)	// 68000 & 68010
			{
				fEnableFPU->SetValue(0);
				fEnableFPU->SetEnabled(false);
				f24BitAddressMode->SetValue(1);
				f24BitAddressMode->SetEnabled(false);
				fCPUCompatible->SetEnabled(cpu == 0 ? true : false);
				fPrefs->cpu_level = cpu;
				fPrefs->address_space_24 = 1;
			}
			else if (cpu == 2)	// 68020
			{
				fEnableFPU->SetValue(fPrefs->cpu_level == 2 ? 0 : 1);
				fEnableFPU->SetEnabled(true);
				f24BitAddressMode->SetValue(fPrefs->address_space_24);
				f24BitAddressMode->SetEnabled(true);
				fCPUCompatible->SetEnabled(false);
				if (fPrefs->cpu_level != 2)
					fPrefs->cpu_level = 3;	// 68020 with FPU
			}
			else	// 68040
			{
				fEnableFPU->SetValue(1);
				fEnableFPU->SetEnabled(false);
				f24BitAddressMode->SetValue(0);
				f24BitAddressMode->SetEnabled(false);
				fCPUCompatible->SetEnabled(false);
				fPrefs->cpu_level = 4;
				fPrefs->address_space_24 = 0;
			}
			break;
		}
		case kMsgCPUSpeed:
		{
			int32 index = popUpIndex(msg);
			
			if (index == 2)
			{
				if (fPrefs->m68k_speed > 0)
					fCPURate->SetValue(fPrefs->m68k_speed);
				else
					fPrefs->m68k_speed = fCPURate->Value();
			}
			else
				fPrefs->m68k_speed = index - 1;
			fCPURate->SetEnabled(fPrefs->m68k_speed > 0 ? true : false);
			break;
		}

		/******** ROM ********/

		case kMsgROMFile:
			OpenFilePanel(fROMFile,kFilePanelModeROM,"UAE: Select ROM file");
			break;
		case kMsgROMKey:
			OpenFilePanel(fROMKey,kFilePanelModeROMKey,"UAE: Select ROM key file");
			break;

		/******** Floppy Disk ********/

		case 'sDF0':	// insert a floppy disk (units 0-3)
		case 'sDF1':
		case 'sDF2':
		case 'sDF3':
		{	
			char title[64];

			fFloppyPanelUnit = (msg->what & 0xff) - '0';
			sprintf(title,"UAE: Select floppy in unit %d (DF%d:)",fFloppyPanelUnit,fFloppyPanelUnit);
			
			OpenFilePanel(fFloppyPath[fFloppyPanelUnit],kFilePanelModeFloppy,title);
			break;
		}
		case 'eDF0':	// eject a floppy disk (units 0-3)
		case 'eDF1':
		case 'eDF2':
		case 'eDF3':
		{
			uint8 unit = (msg->what & 0xff) - '0';

			disk_eject(unit);
			fFloppyPath[unit]->SetText(NULL);
			break;
		}
		case kMsgCreateFloppyFile:
			OpenSaveFilePanel(kFilePanelModeFloppy,"UAE: Save blank floppy disk file");
			break;

		/******** Video ********/

		case kMsgGfxSize:
		{
			int32 index = popUpIndex(msg);
			const int widthAt[] =  {320,640,800,1024,1280};
			const int heightAt[] = {256,480,600, 768,1024};
			
			fPrefs->gfx_width = widthAt[index];
			fPrefs->gfx_height = heightAt[index];
			fPrefs->gfx_lores = fPrefs->gfx_width < 640 ? 1 : 0;
			fPrefs->gfx_linedbl = fPrefs->gfx_height < 400 ? 0 : 1;
			break;
		}
		case kMsgGfxDepth:
		{
			int32 index = popUpIndex(msg);
			switch(index)
			{
				case 0: fPrefs->color_mode = 0; break;
				case 1: fPrefs->color_mode = 1; break;
				case 2: fPrefs->color_mode = 2; break;
				case 3: fPrefs->color_mode = 5; break;
				default:
					switch(gDisplayMode.space)
					{
						case B_CMAP8:
							fPrefs->color_mode = 0;
							break;
						case B_RGB15:
						case B_RGBA15:
							fPrefs->color_mode = 1;
							break;
						case B_RGB16:
							fPrefs->color_mode = 2;
							break;
						default:
							fPrefs->color_mode = 5;
							break;
					}
					break;
			}
			break;
		}
		case kMsgGfxChipset:
		{
			int32 index = popUpIndex(msg);
			
			switch(index)
			{
				case 0:
					fPrefs->chipset_mask = 0;
					break;
				case 1:
					fPrefs->chipset_mask = CSMASK_ECS_AGNUS;
					break;
				case 2:
					fPrefs->chipset_mask = CSMASK_ECS_DENISE;
					break;
				case 3:
					fPrefs->chipset_mask = CSMASK_ECS_DENISE | CSMASK_ECS_AGNUS;
					break;
				default:
				    fPrefs->chipset_mask = CSMASK_AGA | CSMASK_ECS_DENISE | CSMASK_ECS_AGNUS;
				    break;
			}
			break;
		}

		/******** Audio ********/

		case kMsgSoundMode:
		{
			fPrefs->produce_sound = popUpIndex(msg);
			
			bool enable = fPrefs->produce_sound > 1 ? true : false;
			
			fSoundFrequency->SetEnabled(enable);
			fSoundBits->SetEnabled(enable);
			fSoundChannels->SetEnabled(enable);
			fSoundInterpolation->SetEnabled(enable);
			fSoundBuffer->SetEnabled(enable);
			break;
		}
		case kMsgSoundFrequency:
			fPrefs->sound_freq = 11025 << popUpIndex(msg);
			break;
		case kMsgSoundBits:
			fPrefs->sound_bits = 8 * (popUpIndex(msg)+1);

			fSoundInterpolation->SetEnabled(fPrefs->sound_bits == 16);
			break;
		case kMsgSoundChannels:
			fPrefs->stereo = popUpIndex(msg);
			break;
		case kMsgSoundInterpolation:
			fPrefs->sound_interpol = popUpIndex(msg);
			break;

		/******** Input ********/

		case kMsgJoystickPort0:
			SetEnabledRadioButton(fJoystickPort1,fJoystickPort0->ActiveButton());
			break;
		case kMsgJoystickPort1:
			SetEnabledRadioButton(fJoystickPort0,fJoystickPort1->ActiveButton());
			break;
		case kMsgKeyboardLanguage:
			fPrefs->keyboard_lang = (KbdLang)popUpIndex(msg);
			break;

		/******** Harddisk ********/

		case kMsgFileSystemList:
		{
			bool selected = fFileSystemView->CurrentSelection() >= 0;

			fEditFileSystem->SetEnabled(selected);
			fRemoveFileSystem->SetEnabled(selected);
			fUpFileSystem->SetEnabled(selected);
			fDownFileSystem->SetEnabled(selected);
			break;
		}
		case kMsgAddLocalDirectory:
		case kMsgAddHardFile:
		case kMsgEditFileSystem:
		{
			FileSystemItem *item = NULL;
			LocalDirectoryItem *ldItem = NULL;
			HardFileItem *hfItem = NULL;
			int32 index = fFileSystemView->CurrentSelection();
			
			if (msg->what == kMsgEditFileSystem)
				item = dynamic_cast<FileSystemItem *>(fFileSystemView->ItemAt(index));

			BRect rect;
			rect.left = Frame().left+50;	rect.top = Frame().top+50;
			rect.right = rect.left+300;  rect.bottom = rect.top+50;

			MWindow *window = NULL;
			
			if (msg->what == kMsgAddLocalDirectory || (ldItem = dynamic_cast<LocalDirectoryItem *>(item)) != NULL)
				window = new LocalDirectoryWindow(rect,fFileSystemView,ldItem);
			else if (msg->what == kMsgAddHardFile || (hfItem = dynamic_cast<HardFileItem *>(item)) != NULL)
				window = new HardFileWindow(rect,fFileSystemView,hfItem);

			if (window != NULL)
				window->Show();
			break;
		}
		case kMsgUpFileSystem:
		{
			int index = fFileSystemView->CurrentSelection();
			
			fFileSystemView->MoveItem(index,index-1);
			break;
		}
		case kMsgDownFileSystem:
		{
			int index = fFileSystemView->CurrentSelection();
			
			if (index < fFileSystemView->CountItems()-1)
				fFileSystemView->MoveItem(index,index+1);
			break;
		}
		case kMsgRemoveFileSystem:
		{
			FileSystemItem *item = static_cast<FileSystemItem *>(fFileSystemView->ItemAt(fFileSystemView->CurrentSelection()));
			
			fFileSystemView->RemoveItem(item);
			delete item;
			break;
		}

		/******** Menu ********/

		case kMsgMenuNew:
			default_prefs(&currprefs);
			UpdateViews();
			break;
		case kMsgMenuOpen:
			OpenFilePanel(NULL,kFilePanelModeConfig,"UAE: Open a configuration file");
			break;
		case kMsgMenuSave:
		{
			if (gSettingsEntry == NULL)
				break;

			fSaveFilePanelMode = kFilePanelModeConfig;

			BMessage message(B_SAVE_REQUESTED);

			BEntry parent;
			if (gSettingsEntry->GetParent(&parent) == B_OK)
			{
				entry_ref ref;
				if (parent.GetRef(&ref) == B_OK)
					message.AddRef("directory",&ref);
			}
			char name[B_FILE_NAME_LENGTH];
			gSettingsEntry->GetName(name);
			message.AddString("name",name);

			PostMessage(&message);
			break;
		}
		case kMsgMenuSaveAs:
			OpenSaveFilePanel(kFilePanelModeConfig,"UAE: Save configuration file");
			break;

		/******** Misc. ********/

		case kMsgSettingsPage:
		{
			int32 selected = fListView->CurrentSelection();

			if (selected >= 0)
			{
				fPagesView->ActivateLayer(selected);
				
				switch(selected)	// simple drag & drop support
				{
					case 1:
						fFilePanelMode = kFilePanelModeROM;
						fFilePanelControl = fROMFile;
						break;
					case 5:
						fFilePanelMode = kFilePanelModeFloppy;
						fFilePanelControl = fFloppyPath[fFloppyPanelUnit = 0];
						break;
					default:
						fFilePanelMode = kFilePanelModeNone;
						fFilePanelControl = NULL;
						break;
				}
			}
			break;
		}
		case kMsgReset:
			uae_reset();
		case kMsgStart:
			gQuit = false;
		case kMsgQuit:
			PostMessage(B_QUIT_REQUESTED);
			break;
		case B_REFS_RECEIVED:
			RefsReceived(msg);
			break;
		case B_SAVE_REQUESTED:
			SaveRequested(msg);
			break;
		case B_ABOUT_REQUESTED:
		{
			char str[256];
			
			sprintf(str,"Un*x Amiga Emulator\n"
						"\tby Bernd Schmidt and others\n\n"
						"Version %d.%d.%d BeOS Release %d\n\n"
						"BeOS R5 port (mostly rewritten) by Axel Dörfler\n"
						"BeOS R4.5 Port by David Sowsy\n"
						"BeOS R4 MediaKit sound fix by Raphael Moll\n"
				     	"BeBox port by Christian Bauer\n"
				     	"Additional porting by Patrick Hanevold\n\n"
				     	"The Settings window uses Marco Nelissen's liblayout"
				     	" and some code originally written by Brian Tietz.\n",
				     	UAEMAJOR,UAEMINOR,UAESUBREV,kBeUAERelease);
		
			BAlert *alert = new BAlert("",str,"Ok",NULL,NULL,B_WIDTH_FROM_LABEL,B_INFO_ALERT);
			if (alert == NULL)
				return;
			BTextView *view = alert->TextView();
			BFont font;
		
			view->SetStylable(true);
			
			view->GetFont(&font);
			font.SetSize(22);
			font.SetFace(B_BOLD_FACE); 			
			view->SetFontAndColor(0,20,&font);
		
			alert->Go(NULL);
			break;
		}
		default:
			_inherited::MessageReceived(msg);
	}
}


status_t UAESettings::CreateFloppyFile(BFile *file)
{
    off_t fileSize = 880 * 1024;

	return file->SetSize(fileSize);
}


/*************************************************************************************/
// FrameRateSlider is used in the Video/Amiga settings page
// #pragma mark -


FrameRateSlider::FrameRateSlider(int32 min,int32 max)
	: MSlider("Show every",min,max)
{
	SetHashMarkCount(max-min + 1);
	SetHashMarks(B_HASH_MARKS_BOTH);	
}


char *FrameRateSlider::UpdateText() const
{
	const char *nth[] = {"none","single","second","third","fourth"};

	if (Value() < 5)
		sprintf((char *)fStatusText,"%s frame",nth[Value()]);
	else
		sprintf((char *)fStatusText,"%dth frame",Value());

	return((char *)fStatusText);
}


/*************************************************************************************/
// SpeedRateSlider is used in the CPU settings page
// #pragma mark -


SpeedRateSlider::SpeedRateSlider(int32 min,int32 max)
	: MSlider("Speed rate:",min,max), fMin(min), fMax(max)
{
	SetLimitLabels("CPU","Custom Chips");
	SetHashMarkCount(5);
	SetHashMarks(B_HASH_MARKS_BOTH);	
}


char *SpeedRateSlider::UpdateText() const
{
	if (Value() == fMin)
		strcpy((char *)fStatusText,"CPU fastest");
	else if (Value() == fMax)
		strcpy((char *)fStatusText,"CPU lowest");
	else
		sprintf((char *)fStatusText,"%d",Value());

	return((char *)fStatusText);
}


/*************************************************************************************/
// MemorySlider is used in the ROM/RAM and Gfxcard memory settings page
// #pragma mark -


MemorySlider::MemorySlider(int32 min,int32 max,bool allowZero)
	: MSlider("Memory Size:",SizeToValue(min) - (allowZero ? 1 : 0),SizeToValue(max))
{
	char mn[32],mx[32];
	
	MemoryText(mn,allowZero ? 0 : min);  MemoryText(mx,max);
	SetLimitLabels(mn,mx);

	fMin = SizeToValue(min);  fMax = SizeToValue(max);  fAllowZero = allowZero;

	SetHashMarkCount((fMax - fMin) + (allowZero ? 2 : 1));
	SetHashMarks(B_HASH_MARKS_BOTH);	
}


int32 MemorySlider::SizeToValue(int32 mem) const
{
	if (mem == 0)
		return fAllowZero ? fMin-1 : fMin;

	return (int32)(log(mem+1)/M_LN2);
}


int32 MemorySlider::ValueToSize(int32 value) const
{
	if (value < fMin)
		return 0;

	return 1 << value;
}


void MemorySlider::SetMemoryValue(int32 value)
{
	BControl::SetValue(SizeToValue(value));
}


int32 MemorySlider::MemoryValue() const
{
	return ValueToSize(Value());
}


void MemorySlider::MemoryText(char *text,off_t value)
{
	if (value <= kKBSize)
		sprintf(text,"%ld Bytes",value);
	else if (value < kMBSize)
		sprintf(text,"%.1f kB",1.0*value/kKBSize);
	else if (value < kGBSize)
		sprintf(text,"%.1f MB",1.0*value/kMBSize);
	else
		sprintf(text,"%.1f GB",1.0*value/kGBSize);
}


char *MemorySlider::UpdateText() const
{
	MemoryText((char *)fStatusText,MemoryValue());
	return((char *)fStatusText);
}


/*************************************************************************************/
//  ListItems with icons
//	#pragma mark -


static int32 gMaximumWidth = 0,gMaximumHeight = 0;
static int32 gTopOffset = 0;


IconItem::IconItem(char *label,int32 idIcon = kIconNone,int32 idIconSelected = kIconNone)
{
	fLabel = strdup(label);

	fIconBitmap = RetrieveBitmap(idIcon);
	fIconSelectedBitmap = RetrieveBitmap(idIconSelected);
	if (fIconSelectedBitmap == NULL)
		fIconSelectedBitmap = fIconBitmap;
}


IconItem::~IconItem()
{
	free(fLabel);

	delete fIconBitmap;
	delete fIconSelectedBitmap;
}


void IconItem::Update(BView *owner,const BFont *font)
{
	BListItem::Update(owner,font);

	float width = gMaximumWidth + (fLabel ? font->StringWidth(fLabel) : 0.0f);
	
	if (width > Width())
		SetWidth(width);
	
	if (gMaximumHeight > Height())
		SetHeight(gMaximumHeight);

	font_height fontHeight;
	font->GetHeight(&fontHeight);
	gTopOffset = (int32)((gMaximumHeight - fontHeight.ascent - fontHeight.descent - fontHeight.leading)/2 + fontHeight.ascent + 1);
}


void IconItem::DrawItem(BView *owner,BRect rect,bool drawEverything)
{
	BBitmap *bitmap;
	
	owner->PushState();
	
	if (IsSelected())
	{
		//owner->SetLowColor(102,136,187); //tint_color(rgb_color(255,255,255),B_HIGHLIGHT_BACKGROUND_TINT));
		owner->SetLowColor(152,182,212);
		bitmap = fIconSelectedBitmap;
	}
	else
		bitmap = fIconBitmap;

	owner->FillRect(rect,B_SOLID_LOW);
	owner->SetDrawingMode(B_OP_ALPHA);
	if (bitmap != NULL)
		owner->DrawBitmap(bitmap,BPoint(rect.left + (gMaximumWidth - bitmap->Bounds().Width())/2,
										rect.top + (gMaximumHeight - bitmap->Bounds().Height())/2));
	
	if (fLabel != NULL)
		owner->DrawString(fLabel,BPoint(rect.left + gMaximumWidth + 5,rect.top + gTopOffset));
	
	if (!IsEnabled())
	{
		owner->SetHighColor(255,255,255,128);
		owner->FillRect(rect,B_SOLID_HIGH);
	}
	owner->PopState();
}


BBitmap *IconItem::RetrieveBitmap(int32 id)
{
	if (id == kIconNone)
		return NULL;

	BBitmap *bitmap = retrieveBitmap(id);

	if (bitmap != NULL)	// update maximum icon size
	{
		BRect bounds = bitmap->Bounds();
		
		if (gMaximumWidth < bounds.Width())
			gMaximumWidth = (int32)bounds.Width();
		if (gMaximumHeight < bounds.Height()+2)
			gMaximumHeight = (int32)bounds.Height()+2;
	}
	return bitmap;
}


/*************************************************************************************/
//  support functions
//	#pragma mark -

/*
static struct strlist *duplicateUnknownLines(struct strlist *src)
{
	if (!src)
		return NULL;

	struct strlist *list = NULL, *origlist = NULL;

	list = origlist = malloc(sizeof(struct strlist));
	while(src)
	{
		if (src->str)
		{
			list->str = malloc(strlen(src->str) + 1);
			strcpy(list->str,src->str);
		}
		else
			list->str = NULL;

		src = src->next;
		if (src)
		{
			list->next = malloc(sizeof(struct strlist));
			list = list->next;
		}
	}
	list->next = NULL;

	return origlist;
}


static void copyPrefs(struct uae_prefs *dst,struct uae_prefs *src)
{
	free_prefs(dst);

	// Shallow copy first, for all the trivial stuff
	*dst = *src;

	// Now, fix up the linked-lists
	dst->unknown_lines = duplicateUnknownLines(src->unknown_lines);

	dst->mountinfo = dup_mountinfo(src->mountinfo);
}
*/

int32 popUpIndex(BMessage *msg)
{
	int32 index;
	
	if (msg->FindInt32("index",&index) == B_OK)
		return index;
	
	return 0;
}


MView *createFileView(MTextControl **control,char *label,uint32 id)
{
	VGroup *vGroup = new VGroup
	(
		new HGroup
		(
			*control = new MTextControl(label,NULL),
			new MButton("Select" B_UTF8_ELLIPSIS,new BMessage(id),NULL,minimax(-1,-1,30,1E5,1)),
			NULL
		),
		NULL
	);
	vGroup->flags |= M_NO_Y_LEFTOVERS;
	return vGroup;
}


BBitmap *retrieveBitmap(int32 id)
{
	if (id == kIconNone)
		return NULL;

	BBitmap *bitmap = NULL;
	const void *data;
	size_t size;

	if ((data = BApplication::AppResources()->LoadResource(B_BITMAP_TYPE,id,&size)) != NULL)
	{
		BMemoryIO stream(data,size);
		BMessage archived;
	
		if (archived.Unflatten(&stream) == B_OK)
		{
			bitmap = new BBitmap(&archived);
			if (bitmap && bitmap->InitCheck() != B_OK)
			{
				delete bitmap;
				return NULL;
			}
		}
	}
	return bitmap;
}


int openGUI(bool isRunning)
{
	bool  volumeChanged = false;
	float volume = 0.8;

	gSettingsLock = create_sem(1,"UAE Settings Lock");
	//LEDs[0] = LEDs[1] = LEDs[2] = LEDs[3] = false;
	quit_program = 0;
	
	gSettingsWindow = new UAESettings(BRect(80,80,400,300),isRunning);
	if (gSettingsWindow == NULL)
		return 1;

	// stop the noise before the window opens
	if (gSoundPlayer != NULL)
	{
		volume = gSoundPlayer->Volume();
		gSoundPlayer->SetVolume(0.0f);
		volumeChanged = true;
	}

	gSettingsWindow->Show();
	
	// Wait until the Settings window has closed
	acquire_sem(gSettingsLock);

	if (volumeChanged && gSoundPlayer != NULL)
		gSoundPlayer->SetVolume(volume);

	if (gQuit)
		return -2;
	
	return 1;
}


/*************************************************************************************/
//  functions called from UAE
//	#pragma mark -

void target_save_options(FILE *file,struct uae_prefs *p)
{
	fprintf(file,"beos.dynamic_resize=%s\n",/*p->be_*/gBePrefs.dynamic_resize ? "true" : "false");
}

int target_parse_option(struct uae_prefs *p,char *option,char *value)
{
	// returns "1" if an option can be parsed, option without target prefix
	return cfgfile_yesno(option,value,"dynamic_resize",& /*p->be_*/gBePrefs.dynamic_resize);
}


int check_prefs_changed_gfx(void)
{
	return 0;
//    if (changed_prefs.gfx_width != currprefs.gfx_width
//	|| changed_prefs.gfx_height != currprefs.gfx_height)
//	fixup_prefs_dimensions (&changed_prefs);
//
//    if (changed_prefs.gfx_width == currprefs.gfx_width
//	&& changed_prefs.gfx_height == currprefs.gfx_height
//	&& changed_prefs.gfx_lores == currprefs.gfx_lores
//	&& changed_prefs.gfx_linedbl == currprefs.gfx_linedbl
//	&& changed_prefs.gfx_correct_aspect == currprefs.gfx_correct_aspect
//	&& changed_prefs.gfx_xcenter == currprefs.gfx_xcenter
//	&& changed_prefs.gfx_ycenter == currprefs.gfx_ycenter
//	&& changed_prefs.gfx_afullscreen == currprefs.gfx_afullscreen
//	&& changed_prefs.gfx_pfullscreen == currprefs.gfx_pfullscreen)
//	return 0;
//
//    graphics_subshutdown ();
//    currprefs.gfx_width = changed_prefs.gfx_width;
//    currprefs.gfx_height = changed_prefs.gfx_height;
//    currprefs.gfx_lores = changed_prefs.gfx_lores;
//    currprefs.gfx_linedbl = changed_prefs.gfx_linedbl;
//    currprefs.gfx_correct_aspect = changed_prefs.gfx_correct_aspect;
//    currprefs.gfx_xcenter = changed_prefs.gfx_xcenter;
//    currprefs.gfx_ycenter = changed_prefs.gfx_ycenter;
//    currprefs.gfx_afullscreen = changed_prefs.gfx_afullscreen;
//    currprefs.gfx_pfullscreen = changed_prefs.gfx_pfullscreen;
//
//    gui_update_gfx ();
//
//    graphics_subinit ();
//
//    if (! inwindow)
//	XWarpPointer (display, None, mywin, 0, 0, 0, 0,
//		      current_width / 2, current_height / 2);
//
//    notice_screen_contents_lost ();
//    init_row_map ();
//    if (screen_is_picasso)
//	picasso_enablescreen (1);
//    return 0;
}


void gui_fps(int x)
{
}


int gui_init(void)
{
	return openGUI(false);	// UAE is not yet running (not really)
}


void gui_exit(void)
{
}


int gui_update(void)
{
	return 1;
}


