#ifndef BE_SETTINGS_H
#define BE_SETTINGS_H
/***********************************************************/
//  UAE - The Un*x Amiga Emulator
//
//  BeOS port - gui settings
//
//  (c) 2000-2001 Axel Drfler
//
/***********************************************************/

#include <AppKit.h>
#include <InterfaceKit.h>
#include <KernelKit.h>
#include <FilePanel.h> 
#include <StorageKit.h>
#include <List.h>
#include <ListItem.h>

#include <MWindow.h>
#include <MSlider.h>
//#include <stdio.h> 
//#include <stdlib.h> 

const uint32 kMsgMenuNew = 'new ';
const uint32 kMsgMenuOpen = 'open';
const uint32 kMsgMenuSave = 'save';
const uint32 kMsgMenuSaveAs = 'svas';

const uint32 kMsgSettingsPage = 'page';
const uint32 kMsgROMFile = 'roms';				// rom
const uint32 kMsgROMKey = 'rkey';
const uint32 kMsgCPU = 'cput';					// cpu
const uint32 kMsgEnableFPU = 'fpu ';
const uint32 kMsg24BitAddressMode = 'ad24';
const uint32 kMsgCPUSpeed = 'cpus';
const uint32 kMsgGfxSize = 'wnsz';				// gfx
const uint32 kMsgGfxDepth = 'dpth';
const uint32 kMsgGfxChipset = 'cset';
const uint32 kMsgSoundMode = 'sdmd';			// sound
const uint32 kMsgSoundFrequency = 'sdfq';
const uint32 kMsgSoundBits = 'sdbi';
const uint32 kMsgSoundChannels = 'sdch';
const uint32 kMsgSoundInterpolation = 'sdip';
const uint32 kMsgJoystickPort0 = 'jop0';		// input/joystick
const uint32 kMsgJoystickPort1 = 'jop1';
const uint32 kMsgKeyboardLanguage = 'kblg';
const uint32 kMsgCreateFloppyFile = 'crdf';		// floppy
const uint32 kMsgAddLocalDirectory = 'addl';	// filesystem
const uint32 kMsgAddHardFile = 'addh';
const uint32 kMsgEditFileSystem = 'fsed';
const uint32 kMsgRemoveFileSystem = 'fsrm';
const uint32 kMsgUpFileSystem = 'fsup';
const uint32 kMsgDownFileSystem = 'fsdn';
const uint32 kMsgFileSystemList = 'fsls';

const uint32 kMsgStart = 'strt';

//const uint32 kMsgReset = 'rest';
//const uint32 kMsgQuit = 'quit';

// File panel modes
const int8 kFilePanelModeNone = 0;
const int8 kFilePanelModeFloppy = 1;
const int8 kFilePanelModeConfig = 2;
const int8 kFilePanelModeROM = 3;
const int8 kFilePanelModeROMKey = 4;
const int8 kFilePanelModeHardDisk = 5;

const minimax kHorizontal(4,0,4,0);
const minimax kVertical(0,4,0,4);
const ulong kBorderSpacing = 8;
const ulong kSliderSpacing = 8;

class MView;
class MListView;
class MTextControl;
class LayeredGroup;
class MCheckBox;
class MPopup;
class MBorder;
class MRadioGroup;
class MButton;

class MemorySlider;
class FileSystemList;
class BetterScrollView;

/******************************** be_prefs ********************************/
// some be specific prefs values, should be moved to "uae_prefs" in the target
// specific option part.

struct be_prefs
{
	int dynamic_resize;
};

extern struct be_prefs gBePrefs;

/******************************** UAESettings ********************************/

class UAESettings : public MWindow
{
	public:
		UAESettings(BRect frame,bool isRunning);
		~UAESettings();
	
		virtual bool QuitRequested(void);
		virtual void MessageReceived(BMessage *msg);

		void OpenSaveFilePanel(int32 panelMode,char *title);
		void SaveRequested(BMessage *msg);
		void OpenFilePanel(MTextControl *control,int32 panelMode,char *title);
		void RefsReceived(BMessage *msg);

		MView *CreateFloppyView(int unit);
		void SetEnabledRadioButton(MRadioGroup *radioGroup,int inactive);
		void UpdateConfig();
		void UpdateViews();

		static status_t CreateFloppyFile(BFile *file);
		
	private:
		static filter_result MessageDropFilter(BMessage *,BHandler **,BMessageFilter *);
		
		MListView *fListView;
		LayeredGroup *fPagesView;
		BMenuItem *fSaveItem;		

		struct uae_prefs *fPrefs;
		bool fIsRunning;	// true if UAE is already running
		
		MTextControl *fFloppyPath[4];
		char *fFloppyLabel[4];
		
		// rom
		MTextControl *fROMFile,*fROMKey;
//		MCheckBox *fROMAutoConfig;

		// ram
		MemorySlider *fFastMemory,*fChipMemory,*fSlowMemory,*fZ3Memory,*fGfxMemory;
		
		// gfx
		MCheckBox *fGfxNTSC,*fGfxFullScreen,*fGfxResize,*fGfxImmediateBlits,*fGfx32BitBlits;
		MCheckBox *fGfxCorrectAspect,*fGfxCenterHorizontal,*fGfxCenterVertical;
		MPopup *fGfxSize,*fGfxDepth,*fGfxChipset;
		MCheckBox *fPicassoFullScreen;
		MSlider *fGfxFrameRate;
		
		// cpu
		MPopup *fCPU,*fCPUSpeed;
		MSlider *fCPURate;
		MCheckBox *fEnableFPU,*f24BitAddressMode,*fCPUCompatible;

		// sound
		MPopup *fSoundMode,*fSoundFrequency,*fSoundChannels,*fSoundBits,*fSoundInterpolation;
		MemorySlider *fSoundBuffer;
		MBorder *fSoundBorder;
		
		// input
		MRadioGroup *fJoystickPort0,*fJoystickPort1;
		MPopup *fKeyboardLanguage;

		// file system & hardfile
		FileSystemList *fFileSystemView;
		MButton *fEditFileSystem,*fRemoveFileSystem,*fUpFileSystem,*fDownFileSystem;

		// file panels
		BFilePanel *fFilePanel,*fSaveFilePanel;
		MTextControl *fFilePanelControl;
		int8	fFilePanelMode,fSaveFilePanelMode;
		uint8	fFloppyPanelUnit;
};

extern int openGUI(bool isRunning);

/******************************** IconItem ********************************/

const int32 kIconNone = 0;
const int32 kIconRAM = 1;
const int32 kIconSelectedRAM = 2;
const int32 kIconAudio = 3;
const int32 kIconSelectedAudio = 4;
const int32 kIconVideo = 5;
const int32 kIconSelectedVideo = 6;
const int32 kIconHarddisk = 7;
const int32 kIconSelectedHarddisk = 8;
const int32 kIconFloppydisk = 9;
const int32 kIconSelectedFloppydisk = 10;
const int32 kIconInput = 11;
const int32 kIconSelectedInput = 12;


class IconItem : public BListItem
{
	public:
		IconItem(char *label,int32 idIcon = kIconNone,int32 idIconSelected = kIconNone);
		~IconItem();

		void DrawItem(BView *owner,BRect itemRect,bool drawEverything = false);
		void Update(BView *owner,const BFont *font);

	private:
		BBitmap *RetrieveBitmap(int32 id);

		char *fLabel;

		BBitmap *fIconBitmap;
		BBitmap *fIconSelectedBitmap;
};


/******************************** FrameRateSlider ********************************/

class FrameRateSlider : public MSlider
{
	public:
		FrameRateSlider(int32 min,int32 max);
		char *UpdateText() const;
	private:
		char fStatusText[32];
};

/******************************** SpeedRateSlider ********************************/

class SpeedRateSlider : public MSlider
{
	public:
		SpeedRateSlider(int32 min,int32 max);
		char *UpdateText() const;
	private:
		char fStatusText[32];
		int32 fMin,fMax;
};

/******************************** MemorySlider ********************************/

class MemorySlider : public MSlider
{
	public:
		MemorySlider(int32 min,int32 max,bool allowZero);
		char *UpdateText() const;
		
		void SetMemoryValue(int32 value);
		int32 MemoryValue() const;
		int32 ValueToSize(int32 value) const;
		int32 SizeToValue(int32 size) const;
		static void MemoryText(char *text,off_t value);
	private:
		int32	fMin,fMax;
		bool	fAllowZero;
		char	fStatusText[32];
};

/******************************** support functions ********************************/

extern MView *createFileView(MTextControl **control,char *label,uint32 id);
extern BBitmap *retrieveBitmap(int32 id);
extern int32 popUpIndex(BMessage *msg);

#endif
