/***********************************************************/
//  BeUAE - The Be Un*x Amiga Emulator
//
//  BeOS port - simple posix thread/sem replacement
//
//	derived from td-posix/penguin.h, (c) 1997 Berndt Schmidt
//  (c) 2001 Axel Drfler
//
/***********************************************************/

#include <OS.h>

typedef sem_id uae_sem_t;
#define uae_sem_init(id,ignore,initial) *(id) = create_sem(initial,"uae")
#define uae_sem_wait(id) acquire_sem(*(id))
//#define uae_sem_getvalue
/*static __inline__ uae_sem_init(sem_id *id,int manualReset,int initialState)
{
	*id = create_sem();
}*/
#define uae_sem_post(id) release_sem(*(id))
#define uae_sem_trywait(id) acquire_sem_etc(*(id),1,B_RELATIVE_TIMEOUT,0)

typedef union {
    int i;
    uae_u32 u32;
    void *pv;
} uae_pt;

/* These currently require the maximum size to be known at initialization
 * time, but it wouldn't be hard to use a "normal" pipe as an extension once the
 * user-level one gets full.
 * We queue up to chunks pieces of data before signalling the other thread to
 * avoid overhead. */

typedef struct {
    uae_sem_t lock;
    uae_sem_t reader_wait;
    uae_sem_t writer_wait;
    uae_pt *data;
    int size, chunks;
    volatile int rdp, wrp;
    volatile int writer_waiting;
    volatile int reader_waiting;
} smp_comm_pipe;

static __inline__ void init_comm_pipe (smp_comm_pipe *p, int size, int chunks)
{
	p->data = (uae_pt *)malloc (size*sizeof (uae_pt));
	p->size = size;
	p->chunks = chunks;
	p->rdp = p->wrp = 0;
	p->reader_waiting = 0;
	p->writer_waiting = 0;
	
	p->lock = create_sem(1,"uae pipe lock");
	p->reader_wait = create_sem(0,"uae pipe reader wait");
	p->writer_wait = create_sem(0,"uae pipe writer wait");
}

static __inline__ void maybe_wake_reader (smp_comm_pipe *p, int no_buffer)
{
	if (p->reader_waiting
		&& (no_buffer || ((p->wrp - p->rdp + p->size) % p->size) >= p->chunks))
    {
		p->reader_waiting = 0;
		release_sem(p->reader_wait);
    }
}

static __inline__ void write_comm_pipe_pt (smp_comm_pipe *p, uae_pt data, int no_buffer)
{
	int nxwrp = (p->wrp + 1) % p->size;

	if (p->reader_waiting)
	{
		/* No need to do all the locking */
		p->data[p->wrp] = data;
		p->wrp = nxwrp;
		maybe_wake_reader (p, no_buffer);
		return;
    }

	acquire_sem(p->lock);

    if (nxwrp == p->rdp)
    {
		/* Pipe full! */
		p->writer_waiting = 1;
		release_sem(p->lock);
		/* Note that the reader could get in between here and do a
		 * sem_post on writer_wait before we wait on it. That's harmless.
		 * There's a similar case in read_comm_pipe_int_blocking. */
		acquire_sem(p->writer_wait);
		acquire_sem(p->lock);
	}
	p->data[p->wrp] = data;
	p->wrp = nxwrp;
	maybe_wake_reader (p, no_buffer);

	release_sem(p->lock);
}

static __inline__ uae_pt read_comm_pipe_pt_blocking (smp_comm_pipe *p)
{
	uae_pt data;

	acquire_sem(p->lock);
	if (p->rdp == p->wrp)
	{
		p->reader_waiting = 1;
		release_sem(p->lock);

		acquire_sem(p->reader_wait);
		acquire_sem(p->lock);
	}
	data = p->data[p->rdp];
	p->rdp = (p->rdp + 1) % p->size;
	
	/* We ignore chunks here. If this is a problem, make the size bigger in the init call. */
	if (p->writer_waiting)
	{
		p->writer_waiting = 0;
		release_sem(p->writer_wait);
	}
	release_sem(p->lock);
	return data;
}

static __inline__ int comm_pipe_has_data (smp_comm_pipe *p)
{
    return p->rdp != p->wrp;
}

static __inline__ int read_comm_pipe_int_blocking (smp_comm_pipe *p)
{
    uae_pt foo = read_comm_pipe_pt_blocking (p);
    return foo.i;
}

static __inline__ uae_u32 read_comm_pipe_u32_blocking (smp_comm_pipe *p)
{
    uae_pt foo = read_comm_pipe_pt_blocking (p);
    return foo.u32;
}

static __inline__ void *read_comm_pipe_pvoid_blocking (smp_comm_pipe *p)
{
    uae_pt foo = read_comm_pipe_pt_blocking (p);
    return foo.pv;
}

static __inline__ void write_comm_pipe_int (smp_comm_pipe *p, int data, int no_buffer)
{
    uae_pt foo;
    foo.i = data;
    write_comm_pipe_pt (p, foo, no_buffer);
}

static __inline__ void write_comm_pipe_u32 (smp_comm_pipe *p, int data, int no_buffer)
{
    uae_pt foo;
    foo.u32 = data;
    write_comm_pipe_pt (p, foo, no_buffer);
}

static __inline__ void write_comm_pipe_pvoid (smp_comm_pipe *p, void *data, int no_buffer)
{
    uae_pt foo;
    foo.pv = data;
    write_comm_pipe_pt (p, foo, no_buffer);
}

typedef thread_id penguin_id;
#define BAD_PENGUIN -1

static __inline__ int start_penguin(void *(*f) (void *),void *arg,penguin_id *id)
{
	*id = spawn_thread((thread_func)f,"uae penguin",B_NORMAL_PRIORITY,arg);
	resume_thread(*id);

	return 1;
}
#define UAE_PENGUIN_EXIT exit_thread(0)
